package edu.lawrence.androiddirectory

import android.content.Context.MODE_PRIVATE
import android.os.Bundle
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.activity.enableEdgeToEdge
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.layout.wrapContentHeight
import androidx.compose.foundation.text.KeyboardActions
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Add
import androidx.compose.material.icons.filled.Delete
import androidx.compose.material3.Button
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Scaffold
import androidx.compose.material3.SwipeToDismissBox
import androidx.compose.material3.SwipeToDismissBoxState
import androidx.compose.material3.SwipeToDismissBoxValue
import androidx.compose.material3.Text
import androidx.compose.material3.TextField
import androidx.compose.material3.TopAppBar
import androidx.compose.material3.rememberSwipeToDismissBoxState
import androidx.compose.runtime.Composable
import androidx.compose.runtime.key
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.toMutableStateList
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.platform.LocalFocusManager
import androidx.compose.ui.text.input.ImeAction
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.rememberNavController
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import edu.lawrence.androiddirectory.ui.theme.AndroidDirectoryTheme
import java.io.IOException

class MainActivity : ComponentActivity() {
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        enableEdgeToEdge()
        var  initialList : List<Person>
        try {
            val json = openFileInput("people.json").bufferedReader().use { it.readText() }
            val sType = object : TypeToken<List<Person>>() { }.type
            initialList = Gson().fromJson<List<Person>>(json, sType)
        } catch (e: IOException) {
            initialList = listOf(Person(name = "Acacia Ackles",office ="Steitz 131"),
                Person(name = "Joe Gregg",office = "Briggs 413"),
                Person(name = "Kurt Krebsbach",office ="Briggs 411"))
        }

        setContent {
            AndroidDirectoryTheme {
                DirectoryApp(initialList = initialList,modifier = Modifier.fillMaxSize())
            }
        }
    }
}

@Composable
fun DirectoryApp(initialList : List<Person>,modifier: Modifier) {
    val people = remember { initialList.toMutableStateList() }
    val context = LocalContext.current
    val navController = rememberNavController()
    fun saveChanges() {
        val gson = Gson()
        val json : String = gson.toJson(people)
        try {
            context.openFileOutput("people.json", MODE_PRIVATE).use { fos ->
                fos.write(json.toByteArray())
            }
        } catch( e : IOException) {
            e.printStackTrace()
        }
    }
    fun createPerson(name: String,office: String) : Unit {
        val newPerson = Person(name = name,office = office)
        people.add(newPerson)
        navController.navigate("list")
        saveChanges()
    }
    fun removePerson(person : Person) {
        people.remove(person)
        saveChanges()
    }

    NavHost(navController,startDestination="list") {
        composable(route="list") {
            ListScreen(people = people,
                doRemove = {person->removePerson(person)},
                toCreate = { navController.navigate("create") },
                modifier = modifier)
        }
        composable(route="create") {
           CreateScreen(onSave = {name,office -> createPerson(name,office)}, modifier = modifier)
        }
    }
}

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun ListScreen(people : List<Person>, doRemove : (Person)->Unit,toCreate : ()->Unit,modifier: Modifier) {
    Scaffold(topBar = {
        TopAppBar(
            title = {
                Text(
                    "Directory",
                )
            },
            actions = {
                IconButton(onClick = { toCreate() }) {
                    Icon(
                        imageVector = Icons.Default.Add,
                        contentDescription = "Add Person"
                    )
                }
            },
        )
    },
        modifier = Modifier.fillMaxSize())
    { innerPadding ->
        Column(verticalArrangement = Arrangement.Top, modifier = Modifier.fillMaxSize().padding(innerPadding)) {
            people.forEach { person ->
                key(person.id) {
                    PersonItem(person = person, onRemove = doRemove)
                }
           }
        }
    }
}

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun PersonItem(
    person : Person,
    onRemove: (Person) -> Unit
) {
    val dismissState = rememberSwipeToDismissBoxState(
        confirmValueChange = {
            if(it == SwipeToDismissBoxValue.EndToStart) {
                onRemove(person)
                return@rememberSwipeToDismissBoxState true
            }
            return@rememberSwipeToDismissBoxState false
        },
        positionalThreshold = { it * .33f }
    )
    SwipeToDismissBox(
        state = dismissState,
        modifier = Modifier.fillMaxWidth().wrapContentHeight(),
        backgroundContent = { DismissBackground(dismissState) },
        content = {
            Row(modifier = Modifier
                .fillMaxWidth()
                .background(MaterialTheme.colorScheme.background)
                .padding(horizontal = 16.dp,vertical=6.dp)) {
                Text(person.name)
                Spacer(modifier=Modifier.width(20.dp))
                Text(person.office)
            }
        })
}

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun DismissBackground(dismissState: SwipeToDismissBoxState) {
    val color = if (dismissState.dismissDirection == SwipeToDismissBoxValue.EndToStart) {
        Color.Red
    } else Color.Transparent

    Box(
        modifier = Modifier
            .fillMaxWidth()
            .background(color)
            .padding(top=4.dp,bottom=4.dp,end=6.dp),
        contentAlignment = Alignment.CenterEnd
    ) {
        Icon(
            imageVector = Icons.Default.Delete,
            contentDescription = null,
            tint = Color.White
        )
    }
}

@Composable
fun CreateScreen(onSave : (String,String)->Unit, modifier : Modifier) {
    val focusManager = LocalFocusManager.current
    val name = remember { mutableStateOf("") }
    val office = remember { mutableStateOf("") }

    Column(verticalArrangement = Arrangement.Center,horizontalAlignment = Alignment.CenterHorizontally, modifier = modifier.fillMaxSize().padding(10.dp)) {
        Row(horizontalArrangement = Arrangement.Start, verticalAlignment = Alignment.Bottom) {
            Text("Name:")
            Spacer(modifier=Modifier.width(10.dp))
            TextField(value = name.value,
                onValueChange = {newValue : String -> name.value = newValue},
                keyboardOptions = KeyboardOptions(imeAction = ImeAction.Done),
                keyboardActions = KeyboardActions(
                    onDone = { focusManager.clearFocus() })
            )
        }
        Spacer(modifier=Modifier.height(10.dp))
        Row(horizontalArrangement = Arrangement.Start, verticalAlignment = Alignment.Bottom) {
        Text("Office:")
        Spacer(modifier=Modifier.width(10.dp))
        TextField(value = office.value,
            onValueChange = {newValue : String -> office.value = newValue},
            keyboardOptions = KeyboardOptions(imeAction = ImeAction.Done),
            keyboardActions = KeyboardActions(
                onDone = { focusManager.clearFocus() })
        )
    }
        Spacer(modifier=Modifier.height(20.dp))
        Button(onClick = {onSave(name.value,office.value)}) {
                Text("Save")
        }
    }
}
@Preview(showBackground = true)
@Composable
fun DirectoryPreview() {
    AndroidDirectoryTheme {
        DirectoryApp(initialList = listOf(Person(name = "Acacia Ackles",office ="Steitz 131"),
            Person(name = "Joe Gregg",office = "Briggs 413"),
            Person(name = "Kurt Krebsbach",office ="Briggs 411")),modifier = Modifier.fillMaxSize())
    }
}